﻿
Imports System.IO
Imports System.Net

Public Class FTP

    Private FtpReq As FtpWebRequest
    Private FtpUri As Uri
    Private Server As String
    Private Username As String
    Private Pwd As String
    Private Cred As NetworkCredential

    ' Private func As App_Functions = New App_Functions

    ''' <summary>
    ''' FTP Server Instanz erstellen
    ''' </summary>
    ''' <param name="FtpServer">Servername mit Endung (z.B. meinServer.de)</param>
    ''' <param name="User">Benutzername</param>
    ''' <param name="Password">Passwort</param>
    ''' <remarks></remarks>
    Public Sub New(ByVal FtpServer As String, ByVal User As String, ByVal Password As String)

        Server = "ftp://" & FtpServer
        Username = User
        Pwd = Password

        Try
            'Server für Request vorbereiten
            FtpUri = New Uri(Server)
            If (FtpUri.Scheme <> Uri.UriSchemeFtp) Then
                ' func.Logbuch("FEHLER: FTP-Server entspricht nicht dem Schema !")
                MsgBox("FEHLER: FTP-Server entspricht nicht dem Schema !", MsgBoxStyle.Critical, "Fehler")
            Else
                Cred = New NetworkCredential(Username, Pwd)
                FtpReq = FtpWebRequest.Create(Server)
                FtpReq.Credentials = Cred
            End If
        Catch ex As Exception
            MsgBox("FEHLER: FTP-Server -> " & ex.Message, MsgBoxStyle.Critical, "Fehler")
        End Try
    End Sub

    ''' <summary>
    ''' Löscht eine Datei auf dem FTP Server -> True/False
    ''' </summary>
    ''' <param name="Datei">Dateiname mit Endung und Pfad (Pfad/Datei)</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function Del(ByVal Datei As String) As Boolean
        Dim all_right As Boolean = False

        'Server für Request vorbereiten
        Dim FtpFileUri = New Uri(Server & "/" & Datei)
        If (FtpFileUri.Scheme <> Uri.UriSchemeFtp) Then
            MsgBox("FEHLER: FTP-Server entspricht nicht dem Schema !", MsgBoxStyle.Critical, "Fehler")
            all_right = False
        Else
            Try
                'Initialisierung des Requests
                Dim FtpFileReq As FtpWebRequest = FtpWebRequest.Create(FtpFileUri)

                FtpFileReq.Method = WebRequestMethods.Ftp.DeleteFile
                FtpFileReq.Credentials = Cred
                'Response abholen
                Dim FtpResponse As FtpWebResponse = FtpFileReq.GetResponse
                'Nur wenn bis hier alles geklappt hat...
                If FtpResponse.StatusDescription <> "" Then
                    all_right = True
                End If

                FtpResponse.Close()
            Catch ex As Exception
                all_right = False
            End Try
        End If

        Return all_right
    End Function

    ''' <summary>
    ''' Verzeichnis abfragen -> String
    ''' </summary>
    ''' <param name="Server_Path">Pfad der ausgelesen werden soll</param>
    ''' <param name="Result_Typ">Art der Ausgabe: 1-Verzeichnisse und Dateien, 2-Nur Dateien, 3-Nur Verzeichnisse</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function Dir(ByVal Server_Path As String, Optional ByVal Result_Typ As Integer = 2) As String
        Dim result As String = ""
        Dim FtpResponseStream As Stream
        Dim FtpReadStream As StreamReader

        'Server für Request vorbereiten
        Dim FtpFileUri = New Uri(Server + "/" + Server_Path)
        MsgBox(Server + "/" + Server_Path)
        If (FtpFileUri.Scheme <> Uri.UriSchemeFtp) Then
            MsgBox("FEHLER: FTP-Server entspricht nicht dem Schema !", MsgBoxStyle.Critical, "Fehler")
        Else
            'Initialisierung des Requests
            FtpReq = FtpWebRequest.Create(FtpFileUri)

            FtpReq.Credentials = Cred
            FtpReq.UsePassive = False
            FtpReq.Method = WebRequestMethods.Ftp.ListDirectory

            Try
                Dim FtpResponse As FtpWebResponse = FtpReq.GetResponse

                FtpResponseStream = FtpResponse.GetResponseStream

                FtpReadStream = New StreamReader(FtpResponseStream, System.Text.Encoding.UTF8)

                result = FtpReadStream.ReadToEnd

                FtpResponse.Close()
                FtpReadStream.Close()
            Catch ex As Exception
                MsgBox("FEHLER: Konnte FTP-Verzeichnis nicht auslesen !", MsgBoxStyle.Critical, "Fehler")
                result = ""
            End Try

            'Verzeichnisdaten aufbereiten
            If result <> "" Then
                Dim data_array() As String = Split(result, Chr(13) & Chr(10))
                result = ""

                'Alles
                If Result_Typ = 1 Then
                    For i As Integer = 0 To UBound(data_array) - 1 Step 1
                        Try
                            result = result & Mid(data_array(i), (InStr(data_array(i), "/") + 1)) & ";"
                        Catch ex As Exception
                            result = result & ";"
                        End Try
                    Next
                End If

                'Nur Dateien
                If Result_Typ = 2 Then
                    For i As Integer = 0 To UBound(data_array) - 1 Step 1
                        Try
                            If InStr(Mid(data_array(i), InStr(data_array(i), "/")), ".") > 0 Then
                                Try
                                    result = result & Mid(data_array(i), (InStr(data_array(i), "/") + 1)) & ";"
                                Catch ex As Exception
                                    result = result & ";"
                                End Try
                            End If
                        Catch ex As Exception
                        End Try
                    Next
                End If

                'Nur Verzeichnisse
                If Result_Typ = 3 Then
                    For i As Integer = 0 To UBound(data_array) - 1 Step 1
                        Try
                            If InStr(Mid(data_array(i), InStr(data_array(i), "/")), ".") = 0 Then
                                Try
                                    result = result & Mid(data_array(i), (InStr(data_array(i), "/") + 1)) & ";"
                                Catch ex As Exception
                                    result = result & ";"
                                End Try
                            End If
                        Catch ex As Exception
                        End Try
                    Next
                End If
            End If
        End If

        Return result
    End Function

    ''' <summary>
    ''' FTP-Upload -> True/False
    ''' </summary>
    ''' <param name="LocalFileName">Name und Pfad zur lokalen Datei (Laufwerk/Pfad/Datei)</param>
    ''' <param name="ServerFileName">Name und Pfad zur remote Datei (Pfad/Datei)</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function Upload(ByVal LocalFileName As String, ByVal ServerFileName As String) As Boolean
        Dim FileContent() As Byte
        Dim all_right As Boolean = False

        'Server für Request vorbereiten
        Dim FtpFileUri = New Uri(Server & "/" & ServerFileName)
        If (FtpFileUri.Scheme <> Uri.UriSchemeFtp) Then
            MsgBox("FEHLER: FTP-Server entspricht nicht dem Schema !", MsgBoxStyle.Critical, "Fehler")
            all_right = False
        Else
            'Initialisierung des Requests
            Dim FtpFileReq As FtpWebRequest = FtpWebRequest.Create(FtpFileUri)

            FtpFileReq.Credentials = Cred
            FtpFileReq.Method = WebRequestMethods.Ftp.UploadFile
            FtpFileReq.UseBinary = True
            FtpFileReq.UsePassive = False

            Try
                'Bytes der Datei einlesen
                FileContent = My.Computer.FileSystem.ReadAllBytes(LocalFileName)
                'Länge des Arrays ermittlen
                FtpFileReq.ContentLength = FileContent.GetLength(0)
                'Neuen RequestStream öffnen
                Dim FtpRequestStream As Stream = FtpFileReq.GetRequestStream
                'Daten schreiben
                FtpRequestStream.Write(FileContent, 0, FtpFileReq.ContentLength)
                'Stream schließen
                FtpRequestStream.Close()
                'Response abholen
                Dim FtpResponse As FtpWebResponse = FtpFileReq.GetResponse
                'Nur wenn bis hier alles geklappt hat...
                If FtpResponse.StatusDescription <> "" Then
                    all_right = True
                End If

                FtpResponse.Close()
                FtpRequestStream.Close()
            Catch ex As Exception
                MsgBox("FEHLER: Datei-Upload fehlgeschlagen !", MsgBoxStyle.Critical, "Fehler")
                all_right = False
            End Try
        End If

        Return all_right
    End Function

    ''' <summary>
    ''' FTP-Download -> True/False
    ''' </summary>
    ''' <param name="ServerFileName">Name und Pfad zur remote Datei (Pfad/Datei)</param>
    ''' <param name="LocalFileName">Name und Pfad zur lokalen Datei (Laufwerk/Pfad/Datei)</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function Download(ByVal ServerFileName As String, ByVal LocalFileName As String) As Boolean
        Dim all_right As Boolean = False

        Dim FtpFileUri = New Uri(Server + "/" + ServerFileName)
        If (FtpFileUri.Scheme <> Uri.UriSchemeFtp) Then
            MsgBox("FEHLER: FTP-Server entspricht nicht dem Schema !", MsgBoxStyle.Critical, "Fehler")
            all_right = False
        Else
            'Initialisierung des Requests
            Dim FtpFileReq As FtpWebRequest = FtpWebRequest.Create(FtpFileUri)

            FtpFileReq.Credentials = Cred
            FtpFileReq.Method = WebRequestMethods.Ftp.DownloadFile
            FtpFileReq.UseBinary = True
            FtpFileReq.UsePassive = False

            Try
                'FTPReader und FTPWriter
                Dim Response As FtpWebResponse = FtpFileReq.GetResponse()
                Dim FTPWriter As IO.FileStream = New IO.FileStream(LocalFileName, IO.FileMode.Create)
                Dim FTPReader As IO.Stream = Response.GetResponseStream()

                'Daten lesen bis das Ende der Datei erreicht ist
                Using FTPReader
                    Dim Buffer As Byte() = New Byte(2047) {}
                    Dim read As Integer = 0

                    Do
                        read = FTPReader.Read(Buffer, 0, Buffer.Length)
                        FTPWriter.Write(Buffer, 0, read)
                    Loop While Not (read = 0)
                End Using

                'Alles Schließen
                FTPReader.Close()
                FTPWriter.Close()

                'Nur wenn bis hier alles geklappt hat...
                If Response.StatusDescription <> "" Then
                    all_right = True
                End If

                Response.Close()
            Catch ex As Exception
                MsgBox("FEHLER: Datei-Download fehlgeschlagen !", MsgBoxStyle.Critical, "Fehler")
                all_right = False
            End Try
        End If

        Return all_right
    End Function

End Class
